SECTION(7, Tutorial 7)

m5_question([This tutorial teaches you how to create single
dimensional and multi-dimensional arrays of non-objects.  The
non-object types in Java are those which aren't declared inside a
CLASS, so it includes the following types: EM(boolean), EM(char),
EM(int), EM(float) and EM(double).  A helpful convention in Java is
that the non-object types start with a lowercase letter, while object
types start with an uppercase letter, such as for example the
STRING CLASS as an example of an object type.  In addition to
this, two different array initialization syntaxes are presented.])

SUBSECTION(7.1, Single dimensional arrays)

m5_question([BO(Question 7.1:) Here is an example of a convenient one
dimensional array initialization syntax.  Study, compile and run the
following code.  The code CLSS(int)TT([[[]]]) should be read out loud
as EM(int array) indicating that the variable TT(a) is an int array,
also known as an array of ints.  Note that the first value of the for
loop below is zero.  This is because in J.T.W. and Java, the first
index of an array is zero not one.  This convention harks back to the
old days of the EM(C Programming Language) and is used because it is
more efficient in the low level of machine language than counting
arrays from one.  Also note that parenthesis are used to delimit
arrays.  I use this practice because this is the only place in Java
where a semicolon follows a closing parenthesis.  If you don't know
what I am talking about, simply ignore that remark!

m4_begin_indent
NU()VAR INT[] VARI(a) = SQU NUMB(1),NUMB(2),NUMB(3) GLY; EOL
FUR (VAR INT VARI(i)=NUMB(0); LT(i,NUMB(3)); i=i+NUMB(1)) EOL
BEGIN EOL
PD SYSTEM_OUT_PRINTLN(STRI("a[") + i + STRI("]=") + a[i]); EOL
END EOL
m4_end_indent

NU()Due to a design oversight by the creators of Java you cannot use
this syntax to re-initialize an array like so:

m4_begin_indent
NU()a = SQU NUMB(4),NUMB(5),NUMB(6) GLY; COMM(// Compilation error) EOL
m4_end_indent

NU()Luckily there is a way array around this oversight and that is to
use a design pattern where you introduce a temporary variable like so:

m4_begin_indent
NU()VAR INT[] VARI(temp) = SQU NUMB(4),NUMB(5),NUMB(6) GLY; EOL
a = temp; COMM(// Array "a" now holds 4 5 6) EOL
m4_end_indent

NU()Later you will learn why this design pattern is useful for
re-initializing multi-dimensional arrays.])

m5_question([BO(Question 7.2:) Write a FUNCTION TT(print) that
takes an int array argument and prints out the array.  You will need
to use the TT(length) property of the array parameter so your FUNCTION
works with arbitrary sized arrays.  Change the MAIN FUNCTION to what
follows so that it contains a call to the TT(print) FUNCTION.

m4_begin_indent
NU()VAR INT[] VARI(a) = SQU NUMB(1),NUMB(2),NUMB(3) GLY; EOL
print(a); EOL
m4_end_indent

])

m6_answer([BO(Answer 7.2:)

m4_begin_indent
NU()FUNCTION VOID FUNC(print)(INT[] VARI(a)) EOL
BEGIN EOL
PD FUR (VAR INT i=0; LT(i,a.length); i=i+NUMB(1)) EOL
PD BEGIN EOL
PD PD SYSTEM_OUT_PRINTLN(STRI("a[") + i + STRI("]=") + a[i]); EOL
PD END EOL
END EOL
m4_end_indent

])

m5_question([BO(Question 7.3:) Write a FUNCTION with same name as the
previous TT(print) FUNCTION, except that this one should take an
argument that is a CLSS(double)TT([[[]]]), also known as a double array.
Two functions with the same name in the same CLASS is allowed in Java
and the practice of using has a special name that is: EM(FUNCTION name
overloading).  Overloading is only allowed when the two functions with
the same name have different parameters.  When you call an overloaded
FUNCTION J.T.W. and Java looks at the number and types of the arguments a
determines from this which of the overloaded functions to call.
Change the MAIN FUNCTION to what follows so that it initializes an
array of double-precision floating point variables and then calls the
second TT(print) FUNCTION.

m4_begin_indent
NU()VAR DOUBLE[] VARI(b) = SQU NUMB(1.1),NUMB(2.2),NUMB(3.3) GLY; EOL
print(b); EOL
m4_end_indent

])

m6_answer([BO(Answer 7.3:)

m4_begin_indent
NU()FUNCTION VOID FUNC(print)(DOUBLE[] VARI(a)) EOL
BEGIN EOL
PD FUR (VAR INT i=0; LT(i,a.length); i=i+NUMB(1)) EOL
PD BEGIN EOL
PD PD SYSTEM_OUT_PRINTLN(STRI("a[") + i + STRI("]=") + a[i]); EOL
PD END EOL
END EOL
m4_end_indent

])

m5_question([Here is an example of a second initialisation syntax.
For this particular example it is better to use the simpler, earlier
initialisation syntax, but when the size of the array to be created is
to be determined at run-time, then the second syntax should used.  The
next question will show you an example of this.

m4_begin_indent
NU()BEGINMAIN EOL
PD VAR INT[] VARI(a) = NEW INT[NUMB(3)]; EOL
PD COMM(// at this point the array is all zeroes) EOL
PD FUR (VAR INT VARI(i)=NUMB(0); LT(i,NUMB(3)); i=i+NUMB(1)) EOL
PD BEGIN EOL
PD PD a[i] = i; EOL
PD END EOL
PD print(a); EOL
ENDMAIN EOL
m4_end_indent

])

m5_question([BO(Question 7.4:) Write a FUNCTION TT(create) takes one
int argument, the size of the array to create and returns an int array
of that size.  Make it so the TT(i)SUP(th) element of the array is
initialized to TT(i).  Call this FUNCTION from the MAIN FUNCTION like
so:

m4_begin_indent
NU()BEGINMAIN EOL
PD VAR INT[] VARI(a) = create(NUMB(3)); EOL
PD print(a); EOL
ENDMAIN EOL
m4_end_indent

])

m6_answer([BO(Answer 7.4:)

m4_begin_indent
NU()FUNCTION INT[] FUNC(create)(INT VARI(size)) EOL
BEGIN EOL
PD VAR INT[] VARI(result) = NEW INT[size]; EOL
PD FUR (VAR INT VARI(i)=NUMB(0); LT(i,result.length); i=i+NUMB(1)) EOL
PD BEGIN EOL
PD PD result[i] = i; EOL
PD END EOL
PD RETURN result; EOL
END EOL
m4_end_indent

])

m5_question([BO(Question 7.5:) Write a FUNCTION TT(create2) takes one
int argument, the size of the array to create and returns a double
array of that size.  Make it so the TT(i)SUP(th) element of the array
is initialized to TT(i.i), given that LT(i,10).  Why is it not
possible to overload that TT(create) FUNCTION?  Try it and see what
the compiler says.  Call TT(create2) from the MAIN FUNCTION like so:

m4_begin_indent
NU()BEGINMAIN EOL
PD VAR DOUBLE[] VARI(a) = create2(NUMB(3)); EOL
PD print(a); EOL
ENDMAIN EOL
m4_end_indent

])

m6_answer([BO(Answer 7.5:) You cannot overload the TT(create)
METHOD because TT(create) and TT(create2) both have the same
arguments.  Note that you cannot overload based on the return type of
a FUNCTION because, as was discussed in TUTE_HELLO_WORLD you might
choose to ignore the return value of a FUNCTION, and in this case it
would be impossible to determine what FUNCTION you want to call.

m4_begin_indent
NU()FUNCTION DOUBLE[] FUNC(create2)(INT VARI(size)) EOL
BEGIN EOL
PD VAR DOUBLE[] VARI(result) = NEW DOUBLE[size]; EOL
PD FUR (VAR INT VARI(i)=NUMB(0); LT(i,result.length); i=i+NUMB(1)) EOL
PD BEGIN EOL
PD PD result[i] = i + i * NUMB(0.1); EOL
PD END EOL
PD RETURN result; EOL
END EOL
m4_end_indent

])

m5_question([BO(Question 7.6:) Write a FUNCTION TT(doubler) that
takes an int array TT(x) and returns a new int array TT(result) that
is twice as big as TT(x).  Copy TT(x) into TT(result) before you
return it.  The extra elements in the TT(result) should all be zero.])

m6_answer([BO(Answer 7.6:)

m4_begin_indent
NU()FUNCTION INT[] FUNC(doubler)(INT[] VARI(x)) EOL
BEGIN EOL
PD VAR INT[] VARI(result) = NEW INT[x.length * NUMB(2)]; EOL
PD FUR (VAR INT VARI(i)=NUMB(0); LT(i,x.length); i=i+NUMB(1)) EOL
PD BEGIN EOL
PD PD result[i] = x[i]; EOL
PD END EOL
PD RETURN result; EOL
END EOL
m4_end_indent

])

m5_question([BO(Question 7.7:) Change the TT(doubler) FUNCTION so
that every zero in the array TT(result) is set to the value 13.])

m6_answer([BO(Answer 7.7:)

m4_begin_indent
NU()FUNCTION INT[] FUNC(doubler)(INT[] VARI(x)) EOL
BEGIN EOL
PD VAR INT[] VARI(result) = NEW INT[x.length * NUMB(2)]; EOL
PD FUR (VAR INT VARI(i)=NUMB(0); LT(i,x.length); i=i+NUMB(1)) EOL
PD BEGIN EOL
PD PD result[i] = x[i]; EOL
PD END EOL
PD FUR (VAR INT VARI(i)=NUMB(0); LT(i,result.length); i=i+NUMB(1)) EOL
PD BEGIN EOL
PD PD IF (result[i] == NUMB(0)) THEN result[i] = 13; EOL
PD END EOL
PD RETURN result; EOL
END EOL
m4_end_indent

])

SUBSECTION(7.2, Two dimensional arrays)

m5_question([BO(Question 7.8:) Here is an example of a convenient two
dimensional array initialization syntax.  Study, compile and run the
following code.  The code CLSS(int)TT([[[]][[]]]) should be read out
loud as EM(int array array) indicating the variable TT(a) is an EM(int
array array), also known as a EM(two-dimensional array of ints).

m4_begin_indent
NU()BEGINMAIN EOL
PD VAR INT[][] a = SQU SQU NUMB(1),NUMB(2),NUMB(3) GLY SQU NUMB(4),NUMB(5) GLY SQU NUMB(6) GLY GLY EOL
EOL
PD FUR (VAR INT VARI(y)=NUMB(0); LT(y,a.length); y=y+NUMB(1)) EOL
PD BEGIN EOL
PD PD FUR (VAR INT VARI(x)=NUMB(0); LT(x,a[y].length); x=x+NUMB(1)) EOL
PD PD BEGIN EOL
PD PD PD SYSTEM_OUT_PRINT(STRI(" ") + a[y][x]); EOL
PD PD END EOL
PD PD SYSTEM_OUT_PRINTLN(); EOL
PD END EOL
ENDMAIN EOL
m4_end_indent

])

m5_question([BO(Question 7.9:) By copying the pattern of the code
above, do some more overloading of the TT(print) FUNCTION by writing
two new TT(print) functions, one taking a two-dimensional array of
ints, the other taken a two-dimensional array of doubles.  The call
both of these functions from the MAIN FUNCTION.])

m6_answer([BO(Answer 7.9:)

m4_begin_indent
NU()FUNCTION VOID FUNC(print)(INT[][] a) EOL
BEGIN EOL
PD FUR (VAR INT VARI(y)=NUMB(0); LT(y,a.length); y=y+NUMB(1)) EOL
PD BEGIN EOL
PD PD FUR (VAR INT VARI(x)=NUMB(0); LT(x,a[y].length); x=x+NUMB(1)) EOL
PD PD BEGIN EOL
PD PD PD SYSTEM_OUT_PRINT(STRI(" ") + a[y][x]); EOL
PD PD END EOL
PD PD SYSTEM_OUT_PRINTLN(); EOL
PD END EOL
END EOL
FUNCTION VOID FUNC(print)(DOUBLE[][] a) EOL
BEGIN EOL
PD FUR (VAR INT VARI(y)=NUMB(0); LT(y,a.length); y=y+NUMB(1)) EOL
PD BEGIN EOL
PD PD FUR (VAR INT VARI(x)=NUMB(0); LT(x,a[y].length); x=x+NUMB(1)) EOL
PD PD BEGIN EOL
PD PD PD SYSTEM_OUT_PRINT(STRI(" ") + a[y][x]); EOL
PD PD END EOL
PD PD SYSTEM_OUT_PRINTLN(); EOL
PD END EOL
END EOL
m4_end_indent

Here is what your MAIN FUNCTION should look like:

m4_begin_indent
NU()BEGINMAIN EOL
PD VAR INT[][] a = SQU SQU NUMB(1),NUMB(2),NUMB(3) GLY SQU NUMB(4),NUMB(5) GLY SQU NUMB(6) GLY GLY; EOL
PD print(a); EOL
PD VAR DOUBLE[][] b = SQU SQU NUMB(1.0),NUMB(2.0),NUMB(3.0) GLY SQU NUMB(4.0),NUMB(5.0) GLY SQU NUMB(6.0) GLY GLY; EOL
PD print(b); EOL
ENDMAIN EOL
m4_end_indent

])

m5_question([m4_changequote(,) Note that if TT(x) is a two-dimensional
array of ints, then TT(x[i]) is a one dimensional array of ints for
each in the range TT(0 ... x.length-1).  Note that in the above code,
TT(a[0]) is an array of three ints, TT(a[1]) is an array of two ints
and TT(a[2]) is an array of one int.  The reason these sub-arrays are
all of different sizes is to save your computer's precious memory.
For example you can have one sub-array much longer than all of the
others without needing to allocate a whole bunch of memory that will
go unused.  Since TT(a[0]) is an int array, you would naively expect
it to be able to be re-initialized like so:

m4_begin_indent
NU()a[NUMB(0)] = SQU NUMB(4),NUMB(5),NUMB(6),NUMB(7)GLY; EOL
m4_end_indent

NU()m4_changequote(,) so that after this code TT(a[0]) holds the four
element long array 4,5,6 and 7.  But as mentioned above in
Section~SECT(7.1), this doesn't work because of a design oversight by
the creators of Java.  Luckily as mentioned above there is a way
around this oversight and that is to use a temporary variable like so:

m4_begin_indent
NU()VAR INT[] VARI(temp) = SQU NUMB(4),NUMB(5),NUMB(6),NUMB(7)GLY; EOL
a[NUMB(0)] = temp; COMM(// Array "a[0]" now holds 4 5 6 7) EOL
m4_end_indent

NU()m4_changequote(,) Like with one dimensional arrays, there is a second
initialisation syntax for two-dimensional arrays and here it is.
Unlike the above code the sub-arrays TT(a[0]), TT(a[1]) and TT(a[2])
are all of equal size, namely three.

m4_begin_indent
NU()VAR INT[][] VARI(a) = NEW INT[NUMB(3)][NUMB(3)]; EOL
a[NUMB(0)][NUMB(0)] = NUMB(1); a[NUMB(1)][NUMB(0)] = NUMB(2); a[NUMB(2)][NUMB(0)] = NUMB(3); EOL
a[NUMB(0)][NUMB(1)] = NUMB(4); a[NUMB(1)][NUMB(1)] = NUMB(5); EOL
a[NUMB(0)][NUMB(2)] = NUMB(6); EOL
m4_end_indent

])

m5_question([BO(Question 7.10:) Write a FUNCTION TT(create3) and
TT(create4) that takes on int argument TT(size) and returns a two
dimensional array of ints or doubles, respectively.  Make is so that
if TT(a) is the name of the returned array, then TT(a[[[y][x]]]) is
set to the value of TT(x+y).])

m6_answer([BO(Answer 7.10:)

m4_begin_indent
NU()FUNCTION INT[][] FUNC(create3)(INT VARI(size)) EOL
BEGIN EOL
PD VAR INT[][] VARI(a) = NEW INT[size][size]; EOL
PD FUR (VAR INT VARI(y)=NUMB(0); LT(y,size); y=y+NUMB(1)) EOL
PD BEGIN EOL
PD PD FUR (VAR INT VARI(x)=NUMB(0); LT(x,size); x=x+NUMB(1)) EOL
PD PD BEGIN EOL
PD PD PD a[y][x] = x + y; EOL
PD PD END EOL
PD END EOL
PD RETURN a; EOL
END EOL
EOL
FUNCTION DOUBLE[][] FUNC(create4)(INT VARI(size)) EOL
BEGIN EOL
PD VAR DOUBLE[][] VARI(a) = NEW DOUBLE[size][size]; EOL
PD FUR (VAR INT VARI(y)=NUMB(0); LT(y,size); y=y+NUMB(1)) EOL
PD BEGIN EOL
PD PD FUR (VAR INT VARI(x)=NUMB(0); LT(x,size); x=x+NUMB(1)) EOL
PD PD BEGIN EOL
PD PD PD a[y][x] = x + y; EOL
PD PD END EOL
PD END EOL
PD RETURN a; EOL
END EOL
m4_end_indent

])

SUBSECTION(7.3, Three dimensional arrays)

m5_question([BO(Question 7.11:) Using the knowledge you have gained
so far about arrays, create, initialize and print a three dimensional
array of ints.])

m6_answer([BO(Answer 7.11:)

m4_begin_indent
NU()COMM(// create array) EOL
VAR INT[][][] VARI(a) = NEW INT[NUMB(3)][NUMB(3)][NUMB(3)]; EOL
EOL
COMM(// initialize array) EOL
FUR (VAR INT VARI(z)=NUMB(0); LT(z,a.length); z=z+NUMB(1)) EOL
BEGIN EOL
PD FUR (VAR INT VARI(y)=NUMB(0); LT(y,a[z].length); y=y+NUMB(1)) EOL
PD BEGIN EOL
PD PD FUR (VAR INT VARI(x)=NUMB(0); LT(x,a[z][y].length); x=x+NUMB(1)) EOL
PD PD BEGIN EOL
PD PD PD a[z][y][x] = x + y + z; EOL
PD PD END EOL
PD END EOL
END EOL
EOL
COMM(// print array) EOL
FUR (VAR INT VARI(z)=NUMB(0); LT(z,a.length); z=z+NUMB(1)) EOL
BEGIN EOL
PD FUR (VAR INT VARI(y)=NUMB(0); LT(y,a[z].length); y=y+NUMB(1)) EOL
PD BEGIN EOL
PD PD FUR (VAR INT VARI(x)=NUMB(0); LT(x,a[z][y].length); x=x+NUMB(1)) EOL
PD PD BEGIN EOL
PD PD PD SYSTEM_OUT_PRINT(STRI(" ") + a[x][y][z]); EOL
PD PD END EOL
PD PD SYSTEM_OUT_PRINTLN(); EOL
PD END EOL
PD SYSTEM_OUT_PRINTLN(); EOL
END EOL
m4_end_indent

])

m4_define([m4_back_to_somewhere_href],[J.T.W/index.html#7])

