m4_define([m4_title], Nested packages in Java)
m4_define([m4_title_2], Nested packages in Java SUP(m4_free_full))
m4_begin_html

m4_section_star(Abstract)

m4_p1

The title of this article is the outcome of a discussion with
m4_html(AHREF(http://www.cosc.canterbury.ac.nz/warwick.irwin,Warwick
Irwin)) m4_latex(Warwick FOOTNOTE(Irwin,
TT(http://www.cosc.canterbury.ac.nz/warwick.irwin/))) of the
University of Canterbury Department of Computer Science.  This article
investigates how the visibility status of Java package visible
methods, properties and classes could be altered to reflect more
accurately their nesting positions in the hierarchy of packages.  This
would make Java's package visibility modifier almost as general as
equivalent notion in C++ of friendship without the downsides of C++
friendship.  Two design patterns are presented that achieve almost all
of this more general notion of package visibility without having to
re-engineer the Java language.  I gave a talk about this article to
the <i>IEE Short Papers Evening</i> competition but I failed to secure
any winning place in the competition.

m4_p2

m4_dnl This article explains how package visibility in the Java language
m4_dnl of (1) methods and properties and (2) classes could be made more
m4_dnl general and almost as general as friendship in the C++ language
m4_dnl without the downsides of friendship in C++.  Finally a design pattern
m4_dnl is presented that achieves most of the increased generality without
m4_dnl needing to re-engineer the internals of Java.

m4_section(1, The proposed modification)

m4_p1

Given a general operator TT(OP) on C++ and Java classes here are three
properties that TT(OP) can satisfy:

m4_p2

m4_br

m4_begin_padded_table(|l|l|)
TR(TD(Reflexive)  TD2(TT(A OP A) is true for all classes TT(A)))
TR(TD(Symmetric)  TD2(TT(A OP B) implies TT(B OP A) for all classes TT(A) and TT(B)))
TR(TD(Transitive) TD2(TT(A OP B) and TT(B OP C) implies TT(A OP C) for all classes TT(A)[[,]] TT(B) and TT(C).))
m4_end_padded_table

m4_br

m4_p1

The EM(C++ Friendship Operator), henceforth known as TT(OPCPP) is
reflexive but not symmetric or transitive because classes are friends
with themselves is the only thing you can say about a C++ program
without actually looking at the source code to see what class is
friends with what.  Before we can look at the corresponding operator
in Java, we need to look at a sample diagram of the Java source files
and directories of a sample Java project.  In the following diagram,
RED(red) dots represent Java source files and GRN(green)
boxes represent directories:

m4_p2

m4_img(dir)

m4_p1

The following graph shows how the classes of the above diagram relate
in terms of the EM(Java Package Field Visibility Operator), henceforth
known as TT(OPJAVA).  Importantly, we are looking at package
visibility applied to fields (methods and properties) but not to
classes.  Therefore we assume that all classes are declared with
public visibility and not package visibility.  As shown below, each
distinct package is an island into itself because classes in
subdirectories cannot access package visible fields in parent
directories.

m4_p2

m4_img(java)

m4_p1

The TT(OPJAVA) operator is reflexive, symmetric and transitive so it is
less general than TT(OPCPP).  Here is a possible way that TT(OPJAVA) operator
could be generalised:

m4_p2

m4_img(java-prime)

m4_p1

The EM(Generalised Java Can Access Operator), henceforth referred to
as OPGJ is reflexive, EM(weakly symmetric) and transitive.  By weakly
symmetric I mean TT(A OPGJ B) implies TT(B OPGJ A) only when TT(A) and
TT(B) are in the exact same package.  The OPGJ operator gives rise to
EM(Augmented Trees).  By an Augmented Tree I mean a tree augmented
into a directed graph in such a way that if TT(A OPGJ B) and TT(B OPGJ
C) then TT(A OPGJ C) is in the graph.

m4_p2

m4_br

m4_p1

The operators discussed above lie in the following order from least
general to most general: TT(OPJAVA), TT(OPGJ), TT(OPCPP) as the
properties of TT(OPGJ) are a strict subset of the properties of
TT(OPJAVA) and the properties of TT(OPCPP) are a strict subset of the
properties of TT(OPGJ).  Therefore changing the way that Java package
visibility is implemented from TT(OPJAVA) to TT(OPGJ) would make the
language more expressive and closer to the expressiveness of C++.

m4_p2

m4_br

m4_p1

There are two downsides to friendship in C++.  The first downside is
that to achieve a complicated friendship arrangement, many friend
declarations are needed.  The second downside is that to achieve a
complicated friendship arrangement many source files need to be
EM(touched) (i.e. modified) by having friend declarations added to
them.  The extra touching of files has poor ramifications for programs
like TT(make), which generally recompiles files that include files
that have been touched since the last call to TT(make).  Even with
Java modified in the way previously discussed, it would still be
superior to C++ in respect of not needing any friend declarations.

m4_section(2, The present system)

Consider two Java source files.  Here is the first file:

BEGINTT
m4_java_pretty_print(m4-friendship/1-wrong/alpha/A.java)
ENDTT

Here is the second file:

BEGINTT
m4_java_pretty_print(m4-friendship/1-wrong/alpha/beta/B.java)
ENDTT

m4_p1

Compiling the second file gives an error at the lines marked COMM(***)
because the package TT(alpha.beta) is considered to be different to
the package TT(alpha).  This is despite the fact that the second
file is stored in a subdirectory of the directory that the first file
is stored in.  It is natural to follow the analogy established by the
directory structure and to think of package TT(alpha.beta) as being
EM(inside) package TT(alpha).  Taking this idea to its limit we
would expect that code in package TT(alpha.beta) could access methods
and properties in package TT(alpha) which have package
visibility.  What follows is a design pattern that allows one to get
around this flaw so that sub-packages can access methods and properties
from parent packages.

m4_p2

m4_section(3, A design pattern to make package visibility more general)

m4_p1

To bypass this flaw in Java, and achieve a different visibility system
the first file needs to be split into two parts.  Here is the first
part:

m4_p2

BEGINTT
m4_java_pretty_print(m4-friendship/2-right/alpha/AInner.java)
ENDTT

Here is the second part:

BEGINTT
m4_java_pretty_print(m4-friendship/2-right/alpha/AOuter.java)
ENDTT

m4_p1

It is important in the above code that property TT(AInner) is private
so that only the TT(AOuter) class can access it.  The TT(AInner)
property also has no EM(getter) method as having such would violate
encapsulation and allow a EM(back door) into the class by allowing
arbitrary outside access to the class TT(AOuter)'s private methods and
properties.  For a similar reason the TT(AInner) property has no
EM(setter) method.  Here is the second file reworked to make it
compile:

m4_p2

BEGINTT
m4_java_pretty_print(m4-friendship/2-right/alpha/beta/B.java)
ENDTT

m4_p1

The downside of this system is that only a method of the class
TT(AOuter) can call TT(B)'s TT(wibble) method.  Therefore to call a
method in a subpackage, you need to define and call a method in the
class TT(AOuter) that in turn calls the method in the subpackage.  You
can think of the act of calling a method of a class TT(X) with a
reference to the internals of a different class TT(Y) as a kind of
friendship, as by passing a reference to the internals of TT(Y) you
are granting access to private methods and properties of TT(Y) to
methods in TT(X) which is outside of TT(Y).

m4_p2 m4_p1

BOLD(Note:) Care must be taken to where the TT(alpha.aInner) reference
is used, for example if it is stored somewhere else than a local
variable of the TT(wibble) method, then this opens up access to the
internals of the TT(AOuter) class to everywhere the TT(alpha.aInner)
reference can be accessed.

m4_p2

m4_section(4, An simpler but weaker version of the design pattern)

m4_p1

Instead of splitting a class into an EM(inner) and EM(outer)
components, a simpler but limited functionality version of the
above-mentioned system can be achieved.  Consider the following source
file:

m4_p2

BEGINTT
m4_java_pretty_print(m4-friendship/3-right/alpha/A.java)
ENDTT

m4_p1

In the above code, public access to the TT(java.util.Vector) internals
of TT(A) has been granted by the call to the TT(wibble) method from
the TT(main) method of the TT(A) class.  Here is the file that
contains the definition of the TT(wibble) method:

m4_p2

BEGINTT
m4_java_pretty_print(m4-friendship/3-right/alpha/beta/B.java)
ENDTT

m4_p1

Note that under this restricted system the TT(Vector) inside the TT(A)
class cannot be re-bound to another TT(Vector) outside of the TT(A)
class.  It can only be altered by calling the public methods of the
TT(Vector) class.

m4_p2

m4_section(5, Further work)

m4_p1

Extending the above-mentioned design pattern to methods that pass and
return values of different types is left as an exercise for the
reader.  At the present time I haven't worked out how to apply this
design pattern to private or package visibility methods in classes
that inherit from and use properties of other classes.

m4_p2

m4_br

m4_p1

If the developers of Java could be made aware of the limitation of the
Java under the present system, then Java could be re-engineered so
that it no longer has this limitation.

m4_p2

m4_br

m4_p1

Adding support to the more general system of package visibility for
fields would not change the behaviour of Java source files that
presently compile.  All it would do is make certain files that don't
compile into files that do compile.

m4_p2

m4_br

m4_p1

On the other hand if package visibility as applied to classes is to be
reinterpreted, then some compiler trickery will be needed to ensure
that files that compile under the old system give the same result as
under the new system.  As an example of a potential problem is for
code that uses import statements to import a class from a from a
parent package.  Here is a file called TT(Vector.java):

BEGINTT
m4_java_pretty_print(m4-friendship/4-problem/alpha/Vector.java)
ENDTT

Here is the second file, called TT(Main.java):

BEGINTT
m4_java_pretty_print(m4-friendship/4-problem/alpha/beta/Main.java)
ENDTT

m4_p1

Because the TT(Vector) class has package visibility it is not
accessible to the TT(alpha.beta) package.  Therefore the compiler
creates a new TT(java.util.Vector) at the line marked
COMM(***). Changing TT(javac) to get around this design flaw should
not change the behaviour of classes that do compile under the current
system.  Therefore there needs to be a special case for classes
defined with package visibility.

m4_p2

m4_br

m4_p1

I hope that the cost of this burden in terms of additional compiler
complexity is less than the value of the benefit obtained by the
programmer by extending Java in this way.

m4_p1

m4_define([m5_link],tracker-statcounter-npij.hts.m4)

m4_end_html
