
m4_ifdef([m5_1_true_that],,          [m4_error(true that undefined)])
m4_ifdef([m5_2_page],,               [m4_error(page undefined)])
m4_ifdef([m5_3_stylesheet_location],,[m4_error(stylesheet location undefined)])
m4_ifdef([m5_4_note],,               [m4_error(note undefined)])

m4_ifdef([m5_5_index_part_html],,    [m4_error(index_part_html undefined)])
m4_ifdef([m5_6_index_full_hts],,     [m4_error(index_full_hts undefined)])
m4_ifdef([m5_7_index_part_hts],,     [m4_error(index_part_hts undefined)])
m4_ifdef([m5_8_output],,             [m4_error(output undefined)])

H2(Introduction to Cascading Style Sheets)

<p>

  Recall from SECT_FONT_TAGS that you cannot specify the background
  colour of text using TAG(font) tags.  To do this you will need to
  use Cascading Style Sheets (or C.S.S. for short).  Cascading Style
  Sheets are also useful because it allows you to separate the logic
  of your H.T.M.L. code from the layout (which is purely cosmetic).
  m5_1_true_that.


</p>

<p>

  Although C.S.S. code can legally be put in every m5_2_page file, it is
  better to factor it out of the m5_2_page file and have one C.S.S. file
  that applies to all m5_2_page files.  This is the approach that is
  taken in the Website associated with this tutorial.  Therefore when
  you want to change the C.S.S. file, you only need to do it in one
  place: FCOD(m5_3_stylesheet_location) and the changes will be
  reflected in every Web page of the Website.

</p>

<p>

  To access the file FCOD(stylesheet.css) in every Web page of your
  Website each H.T.M.L. Web page will need to look like this:

</p>

m4_begin_center
<pre>
HQ(<html>)
HQ(<head>)
   <b>&lt;link rel="stylesheet" href="M4_HOME/stylesheet.css" type="text/css"&gt;</b>
HQ(</head>)
HQ(<body>)
   EM(Web page code goes here...)
HQ(</body>)
HQ(</html>)
</pre>
m4_end_center

<p>

  m5_4_note.  Visit the following Website for more detailed tutorials
  on how to use Cascading Style Sheets:
  WEBSITE(www.w3schools.com/css/default.asp).

</p>

H3(Span tags)

m4_ifelse(m4_section,7,[m4_sub_sects_equal(SECT_CSS_1)],[m4_sub_sects_equal(SECT_CSS_2)])


<p>
  Span tags provide a way for you to specify the background colours of
  text.  Here is an example of the definition of a span tag in the
  file FCOD(stylesheet.css):

</p>

m4_begin_center
<pre>
span.rb
{
   font-family: courier;
   font-weight: bold
   font-style: italic;
   color: #ff0000;
   background: #000000;
}
</pre>
m4_end_center

<p>
  With this code in place you can create text which is in the courier
  font, with bold and italics turned on, has a black background and a
  red foreground.  You cannot do this with the TAG(font) tag.  Here
  is how to use this font in an H.T.M.L. document:
</p>

<br>
m4_begin_center
HQ(<span class="rb">I have a red foreground and a black background.</span>)
m4_end_center
<br>

<p>
  Note that all of the entries in the definition of the span tag are
  optional.  When specifying colours such as the TT(color:) or
  TT(background:) attributes, you can use hexadecimal RGB colour
  triplets (as is done above), or you can name colours explicitly
  using a <a
  href="http://coolnerds.com/HTML/colors/colorlistalpha.htm">colour
  name</a> such as TT(red).  The third way to do this is by using a
  RGB decimal triplet such as TT(rgb(255,0,0);).  Legal values for the
  TT(font-family:) attribute include: TT(Times), TT(Courier),
  TT(San-Serif) (also known as TT(Helvetica)) and TT(Comic Sans MS).

</p>

m4_define(AND_FINALLY, and finally open FCOD(m5_5_index_part_html) in a Web
browser to verify that TAG($1) tags are correctly rendered.)

<p>

  m5_question([Add code to FCOD(stylesheet.css) to create a span tag
  with class name TT(myspan) with no bold, no italics, Helvetica font,
  green background and yellow foreground.  Add some H.T.M.L. code to
  FCOD(m5_6_index_full_hts) so that some text is shown with the span
  class TT(myspan) AND_FINALLY(span)])

</p>

<p>

  m6_answer([Here is the definition of the TAG(span) tag in the C.S.S. file:

</p>

<pre>
span.myspan
{
   font-family: Helvetica;
   color: #00ff00;
   background: #ffff00;
}
</pre>

<p>
  Here is some H.T.M.L. code to show text in the TT(myspan) TAG(span) tag:
</p>

<pre>
HTML_QUOTE(<span class="myspan">This text is Helvetica green with a yellow background.</span>)
</pre>

<p>
  ])
</p>

<hr>

H4(Multiple values for the TT(font-family:) attribute)

<p>

  When specifying a font like Comic Sans MS which may not exist in
  non-Windows computers, it is wise to give one or more alternative
  fonts in case the first one doesn't exist.  Consider the following
  C.S.S. code.

</p>

m4_begin_center
<pre>
span.comic
{
   font-family: STRI('Comic Sans MS'), Helvetica;
}
</pre>
m4_end_center

<p>

  In the above, note that Comic Sans MS needs to be quoted as it has
  spaces in it.  Then consider the following H.T.M.L. code:

</p>

m4_begin_center
HTML_QUOTE(<span class="comic">I might be in the Comic Sans MS font.</span>)
m4_end_center

<p>

  The above sentence will appear in the Comic Sans MS font, and if
  that font doesn't exist, it will appear in the Helvetica font.

</p>

H3(TAG(h1) - TAG(h6) tags)

<p>

  Here is a definition of the TAG(h1) tag to go in FCOD(stylesheet.css):

</p>

m4_begin_center
<pre>
h1 {
   font-family: courier;
   font-weight: bold
   font-style: italic;
   color: #ff0000;
   background: #ffffff;
   text-align: right;
   font-size: 40pt
}
</pre>
m4_end_center

<p>
  With the above definition in place, whenever you write a pair of
  tags TAG(h1) DOTS TAG(/h1) it will show up in courier face, bold,
  italic, red foreground, black background, right text alignment and
  font size 40pt.
</p>
<p>

  m5_question([Using similar code to the above, write a C.S.S. definition
  so that TAG(h2) tags show up with center alignment and green
  foreground and no other styles.  Change FCOD(m5_7_index_part_hts) so
  that it contains an TAG(h2) tag AND_FINALLY(h2)])

</p>
<p>

  m6_answer([Here is the definition of the TAG(h2) tag in the C.S.S. file:

</p>
<pre>
h2 {
   text-align: center;
   color: #00ff00;
}
</pre>

<p>
  ])
</p>

<p>

  m5_question([Write a C.S.S. definition so that TAG(h3) tags show up
  with left alignment, green foreground and a yellow background and no
  other styles.  Change FCOD(m5_7_index_part_hts) so that it contains
  an TAG(h3) tag AND_FINALLY(h3)])

</p>
<p>

  m6_answer([Here is the definition of the TAG(h3) tag in the C.S.S. file:

</p>
<pre>
h3 {
   text-align: left;
   color: #00ff00;
   background: #ffff00;
}
</pre>

<p>
  ])
</p>
<p>

  m5_question([Write a C.S.S. definition so that TAG(h4) tags show up
  with left alignment, green foreground, yellow background, orange
  left borders, orange bottom borders and no other styles.  The
  following code will help you to achieve this:

</p>

m4_begin_center
<pre>
 border-bottom:   medium solid rgb(255, 128, 0);
 border-left:     medium solid rgb(255, 128, 0);
</pre>
m4_end_center

<p>
  AND_FINALLY(h4).
  ])
</p>
<p>

  m6_answer([Here is the definition of the TAG(h4) tag in the C.S.S. file:

</p>
m4_begin_indent
<pre>
h4 {
   text-align: left;
   color: #00ff00;
   background: #ffff00;
   border-bottom:   medium solid rgb(255, 128, 0);
   border-left:     medium solid rgb(255, 128, 0);
}
</pre>
m4_end_indent
<p>
  ])
</p>

<hr>

H3(TAG(p) tags)

Here are some attributes that you can apply to default text:

<pre>
p {
   font-family: courier;
   font-weight: bold
   font-style: italic;
   color: #ff0000;
   background: #ffffff;
   text-align: right;
   font-size: 40pt
}
</pre>

<p>

  Note that you will need to use TAG(/p) tags to get this feature to
  work.  In these tutorials I have only taught you to use TAG(p) tags by
  themselves.

</p>
<p>

  m5_question([Make it so that default text appears with a red colour,
  in the Helvetica face with no bold and no italic.])

</p>

<p>

  m6_answer([

</p>

m4_begin_indent
<pre>
p {
   color: red;
   font-family: Helvetica;
   font-style: normal;
   font-weight: normal;
}
</pre>
m4_end_indent

<p>
  ])
</p>

<p>

  m5_question([Visit the following Website
  WEBSITE(www.w3schools.com/css/tryit.asp?filename=trycss_text-transform)
  to find out how to uppercase, lowercase and capitalise text.])

</p>

<p>

  m5_question([Visit the following
  WEBSITE(www.w3schools.com/css/tryit.asp?filename=trycss_text-indent)
  to find out how to set the amount of paragraph indent.])

</p>

<hr>

H3(Removing the underlining of hyperlinks)

<p>

  The following C.S.S. code removes the underlining of hyperlinks:

</p>

m4_begin_center
<pre>
a {text-decoration: none}
</pre>
m4_end_center

<p>

  m5_question([Add this code to the C.S.S. page of your Website
  FCOD(stylesheet.css) and check that it does indeed remove the
  underlining of hyperlinks.])

</p>

<hr>

H3(Background tags)

m4_ifelse(m4_section,7,[m4_sub_sects_equal(SECT_BACKGROUND_TAGS_1)],[m4_sub_sects_equal(SECT_BACKGROUND_TAGS_2)])

<p>

  Here is some code to add to the C.S.S. to give you a stationary
  background image:

</p>

m4_begin_center
<pre>
body
{
   background-color: #ffffff;
   background-image: url(STRI('dolphin.jpg'));
   background-repeat: no-repeat;
   background-attachment: fixed
}
</pre>
m4_end_center

<p>

  m5_question([Write some C.S.S. code so that the background image in
  your Website is different to the one above, and does repeat, but is
  not fixed, so that it scrolls whenever you scroll the screen.  See
  the following Website:
  WEBSITE(www.w3schools.com/css/css_background.asp) for more
  information.])

</p>

<hr>

m4_ignore([

H2(Creating an icon image for your Website)

<p>
  Mozilla Firefox can show a Web shortcut icon next to the location
  bar.  The following code in the file
  FCOD(tutorial-7/src/site/index.m4) should help you to do this:
</p>

m4_begin_center
<pre>
<b>&lt;link rel="shortcut icon" href="M4_HOME/icon-16x16.png"&gt;</b>
</pre>
m4_end_center

])



